import React from 'react';
import {
  scaleTime,
  extent,
  scaleLinear,
  max,
  line,
  timeFormat,
  scaleLog
} from 'd3';

import { XAxis } from './XAxis';
import { YAxis } from './YAxis';

const xValue = (d) => d.date;
const yValue = (d) => d.deathTotal;

const margin = {
  top: 40,
  right: 20,
  bottom: 60,
  left: 80,
};

const formatDate = timeFormat('%b %d');

export const LineChart = ({ data, width, height }) => {
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const allData = data.reduce(
    (accumulator, countryTimeseries) => accumulator.concat(countryTimeseries), 
    []
   );
  
  const epsilon = 1;
  
  const xScale = scaleTime()
    .domain(extent(allData, xValue))
    .range([0, innerWidth]);

  const yScale = scaleLog()
    .domain([epsilon, max(allData, yValue)])
    .range([innerHeight, 0]);

  const lineGenerator = line()
    .x((d) => xScale(xValue(d)))
    .y((d) => yScale(epsilon + yValue(d)));

  const mostRecentDate = xScale.domain()[1];

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        <XAxis xScale={xScale} innerHeight={innerHeight} />
        <YAxis yScale={yScale} innerWidth={innerWidth} />
        {
          data.map(countryTimeseries => (
          	<path 
              stroke={`rgb(${Math.random()*255}, ${Math.random()*255}, ${Math.random()*255})`}
              d={lineGenerator(countryTimeseries)} />
          ))
        }
        <text>Global Coronavirus Deaths Over Time</text>
        <text className="axis-label" transform={`translate(-45, ${innerHeight/2}) rotate(-90)`} text-anchor="middle">Cumulative Deaths</text>
        <text className="axis-label" transform={`translate(${innerWidth/2}, ${innerHeight + 30})`} alignment-baseline="hanging" text-anchor="middle">Time</text>
      </g>
    </svg>
  );
};
